//############################################################################
//
// LaserBoy !!!
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
// Copyright 2003, 2004, 2005, 2006, 2007, 2008, 2009 James Lehman.
// This source is distributed under the terms of the GNU General Public License.
//
// LaserBoy_FLTK_GUI.hpp is part of LaserBoy.
//
// LaserBoy is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// LaserBoy is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with LaserBoy. If not, see <http://www.gnu.org/licenses/>.
//
//############################################################################
#ifndef __LASERBOY_FLTK_GUI_DEFINITIONS__
#define __LASERBOY_FLTK_GUI_DEFINITIONS__

//############################################################################
#include "LaserBoy_GUI_base.hpp"

#include <FL/Fl.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Button.H>
#include <Fl/fl_draw.H>
#include <FL/fl_ask.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Input_.H>
#include <FL/Fl_Int_Input.H>
#include <FL/Fl_Float_Input.H>

#if !defined(WIN32) && !defined(__APPLE__)
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/extensions/Xdbe.h>
#endif

//############################################################################
enum what_menu
{
    in_main_menu = 0,
    in_animate_forward,
    in_animate_reverse,
    in_rotate_forward,
    in_rotate_reverse,
    in_animate_rotate_forward,
    in_animate_rotate_reverse,
    in_system_settings_menu,
    in_signal_if_not_z_menu,
    in_dots_setting_menu,
    in_bit_resolution_menu,
    in_visuals_settings_menu,
    in_switch_settings_menu,
    in_apply_view_menu,
    in_palette_transforms_menu,
    in_frame_transforms_menu,
    in_frame_set_transforms_menu,
    in_draw_color_blank_menu,
    in_draw_text_menu,
    in_draw_circular_arc_menu,
    in_draw_elliptical_arc_menu,
    in_move_scale_rotate_menu,
    in_wave_utilities_menu,
    in_wave_show_settings_menu,
    in_show_wave_options_menu,
    in_set_wave_offsets_menu,
    in_set_signal_polarities_menu,
    in_clear_wave_polarities_menu,
    in_black_level_to_wave_menu,
    in_bit_resolution_to_wave_menu,
    in_join_XY_r_g_b_i_waves_menu,
    in_join_XY_rg_b_waves_menu,
    in_join_XY_rg_b_LR_waves_menu,
    in_add_audio_to_wave_menu,
    in_file_open_menu,
    in_ild_file_open_menu,
    in_dxf_file_open_menu,
    in_wav_file_open_menu,
    in_wav_qm_open_menu,
    in_wav_unformatted_open_menu,
    in_txt_file_open_menu,
    in_bmp_file_open_menu,
    in_save_as_file_menu,
    in_save_as_ild_menu,
    in_save_as_dxf_menu,
    in_save_as_wav_menu,
    in_save_as_txt_menu,
    in_save_as_bmp_menu,
    in_process_terminate_request,
    in_Laserboy_terminating
};

//############################################################################
class lb_box : public Fl_Box
{
public:
    lb_box(int X, int Y, int W, int H)
          : Fl_Box(X, Y, W, H)
            {
                image_data = NULL;
            }
    //------------------------------------------------------------------------
    void set_data(uchar *new_image_data, int W, int H, int D)
            {
                if(!new_image_data)
                    return;
                iw = W;
                ih = H;
                id = D;
                if(image_data)
                    delete [] image_data;
                image_data = new_image_data;
            }
    //------------------------------------------------------------------------
protected:
    void draw()
            {
                if(!image_data)
                    return;

                fl_push_clip(x(), y(), iw, ih);
                fl_draw_image(image_data, x(), y(), iw, ih, id);
                fl_pop_clip();
            }
    //------------------------------------------------------------------------
    int iw, ih, id;
    uchar *image_data;
    //------------------------------------------------------------------------
};

//############################################################################
class LaserBoy_FLTK_GUI : public LaserBoy_GUI_base, public Fl_Double_Window
{
public:
    LaserBoy_FLTK_GUI(int x, int y, int argc, char *argv[]);
    //------------------------------------------------------------------------
virtual
   ~LaserBoy_FLTK_GUI();
    //------------------------------------------------------------------------
virtual  void    display_space();
    //------------------------------------------------------------------------
virtual  string  display_prompt_and_echo_name    (string prompt);
//virtual  string  display_prompt_and_echo_string  (string prompt);
virtual  double  display_prompt_and_echo_double  (string prompt);
virtual  int     display_prompt_and_echo_int     (string prompt);
virtual  int     display_prompt_and_echo_nibble  (string prompt);
//virtual  bool    display_prompt_and_echo_bool    (string prompt);
virtual  void    display_error                   (string err   );
    //------------------------------------------------------------------------
    // fltk_key menu system
    //------------------------------------------------------------------------
    void    check_for_FLTK_arrow_keys  (int key);
    void    animate_forward            ();
    void    animate_reverse            ();
    void    rotate_forward             ();
    void    rotate_reverse             ();
    void    animate_rotate_forward     ();
    void    animate_rotate_reverse     ();
    void    system_settings_menu       ();
    void    signal_if_not_z_menu       ();
    void    dots_setting_menu          ();
    void    bit_resolution_menu        ();
    void    visuals_settings_menu      ();
    void    switch_settings_menu       ();
    void    apply_view_menu            ();
    void    palette_transforms_menu    ();
    void    frame_transforms_menu      ();
    void    frame_set_transforms_menu  ();
    void    draw_color_blank_menu      ();
    void    draw_text_menu             ();
    void    draw_circular_arc_menu     ();
    void    draw_elliptical_arc_menu   ();
    void    move_scale_rotate_menu     ();
    void    wave_utilities_menu        ();
    void    wave_show_settings_menu    ();
    void    show_wave_options_menu     ();
    void    set_wave_offsets_menu      ();
    void    set_signal_polarities_menu ();
    void    clear_wave_polarities_menu ();
    void    black_level_to_wave_menu   ();
    void    bit_resolution_to_wave_menu();
    void    join_XY_r_g_b_i_waves_menu ();
    void    join_XY_rg_b_waves_menu    ();
    void    join_XY_rg_b_LR_waves_menu ();
    void    add_audio_to_wave_menu     ();
    void    file_open_menu             ();
    void    ild_file_open_menu         ();
    void    dxf_file_open_menu         ();
    void    wav_file_open_menu         ();
    void    wav_qm_open_menu           (bool append);
    void    wav_unformatted_open_menu  (bool append);
    void    txt_file_open_menu         ();
    void    bmp_file_open_menu         ();
    void    save_as_file_menu          ();
    void    save_as_ild_menu           ();
    void    save_as_dxf_menu           ();
    void    save_as_wav_menu           ();
    void    save_as_txt_menu           ();
    void    save_as_bmp_menu           ();
    void    process_terminate_request  ();
    void    Laserboy_terminating       ();
    void    main_menu                  ();
    int     start_menu_loop            ();
    //------------------------------------------------------------------------
    bool    display_wave               ();
    bool    display_unframed_wave      ();
    void    display_LaserBoy_wave      (fstream& in, LaserBoy_wave_header header);
    void    display_unframed_wave      (fstream& in, LaserBoy_wave_header header);
    //------------------------------------------------------------------------
    int     handle(int event);
    //------------------------------------------------------------------------
    int     fltk_key,
            current_menu;
            
    char    formatted_string[80];

    uchar  *image_data;

    lb_box *box;

    Fl_File_Chooser *file_chooser;
    //------------------------------------------------------------------------
};

//############################################################################
#endif

//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################
